<?php

namespace Modules\Affiliate\Entities;

use App\Models\Order;
use App\Models\OrderDetail;
use App\Models\User;
use App\Traits\ModelTrait;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Modules\Affiliate\Services\AffiliateHelper;
use Illuminate\Support\Facades\Session;
use Modules\Affiliate\Services\ReferralMailService;

class ReferralPurchase extends Model
{
    use ModelTrait;
    protected $fillable = ['affiliate_user_id', 'order_id', 'commission', 'user_id', 'medium', 'status'];

    public function affiliateUser()
    {
        return $this->belongsTo(AffiliateUser::class, 'affiliate_user_id', 'id');
    }

    public function order()
    {
        return $this->belongsTo(Order::class, 'order_id', 'id');
    }

    public function user()
    {
        return $this->belongsTo(User::class, 'user_id', 'id');
    }

    public function decrementCommission($value = 0): void
    {
        $this->decrement('commission', $value);
    }

    /**
     * referral user commission from purchase
     *
     * @param $latestOrder
     * @param $detailsOrder
     * @return int|true
     */
    public static function purchase($latestOrder = null, $detailsOrder = [])
    {
        $helper = AffiliateHelper::getInstance();
        $user = $helper->getUserData();
        $selfRefer = $helper->checkSelfRefer($user);
        $data = [];

        if (!is_null($user) && $selfRefer && $user != "null") {
            $user = json_decode($user, true);

            if (isset($user['id'])) {
                $userDetails = AffiliateUser::where('id', $user['id'])->activeUser();

                if (!$userDetails->exists()) {
                    $helper->destroy();
                    $helper->destroyCookie();
                    return 0;
                }
            }

            $commissions = (new CommissionPlan)->getCommission($user, $latestOrder, $detailsOrder);

            foreach ($commissions['commissionData'] as $commission) {

                if (isset($commission['commission']) && $commission['commission'] > 0) {
                    $data[] = [
                        'affiliate_user_id' => $commission['affiliate_user_id'],
                        'order_id' => $latestOrder->id,
                        'commission' => $commission['commission'],
                        'user_id' => $latestOrder->user_id,
                        'medium' => Session::has('coupon_referral') ? 'Coupon' : 'Link',
                        'status' => 'Hold',
                    ];
                }
            }

            if (count($data) > 0) {
                self::store($data);
                Session::forget('coupon_referral');
                CommissionLog::store($commissions['commission_logs'], $latestOrder->id);

                return true;
            }
        }

        return 0;
    }

    /**
     * store
     *
     * @param $data
     * @return bool
     */
    public static function store($data = [])
    {
        if (parent::insert($data)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * referral purchase update
     *
     * @param $orderId
     * @param $orderStatus
     * @return void
     */
    public static function referralPurchaseUpdate($orderId, $orderStatus)
    {
        if ($orderStatus->payment_scenario == 'paid') {
            $purchaseData = parent::where('order_id', $orderId)->get();
            $oneOnly = false;

            foreach ($purchaseData as $data) {
                if ($data->status == 'Hold') {
                    $data->status = 'Approve';
                    $data->save();
                    $data->affiliateUser->incrementNetCommission($data->commission);

                    if (!$oneOnly) {
                        $orderTotalAmount = $data->order?->total - $data->order?->shipping_charge;
                        $oneOnly = true;
                        $data->affiliateUser->incrementRevenue($orderTotalAmount);
                        Referral::userProductPurchaseUpdate($data->affiliate_user_id, $data->order?->orderDetails);
                    }
                }
            }

            // check if any product in an order failed or undelivered
            if ($oneOnly) {
                $orderDetails = OrderDetail::where('order_id', $orderId)->get();
                foreach ($orderDetails as $detail) {
                    if ($detail->orderStatus?->payment_scenario != 'paid') {
                        CommissionLog::decrementCommission($detail->product_id, $detail);
                    }
                }
            }

        } elseif ($orderStatus->slug == 'cancelled' || $orderStatus->slug == 'failed') {
            $purchaseData = parent::where('order_id', $orderId)->get();

            foreach ($purchaseData as $data) {
                if ($data->status == 'Hold') {
                    $data->status = 'Rejected';
                    $data->save();
                }
            }

        }
    }
}
